/*

sig.h

Author: Liraz Siri <liraz@bigfoot.com>

Copyright (c) 1999 Liraz Siri <liraz@bigfoot.com>, Ariel, Israel
              All rights reserved.

Created: Sun Oct  3 16:05:50 IST 1999

A thin layer above the signal API
*/
#include <signal.h>

#define sig_pending(s)  sig_ispending(s)
#define sig_blocked(s)  sig_isblocked(s)

void    sig_handleall   ();
void    sig_blockall    ();
void    sig_unblockall  ();
void    sig_handle      (int signum);
void    sig_ignore      (int signum);

int     sig_block       (int signum);
int     sig_unblock     (int signum);
int     sig_isblocked   (int signum);
int     sig_ispending   (int signum);
